/*
 * delay() for the Raspberry Pi Pico RP2040
 *
 * Copyright (c) 2021 Earle F. Philhower, III <earlephilhower@yahoo.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <pico.h>
#include <pico/time.h>
#include "Arduino.h"

extern "C" void delay( unsigned long ms ) {
    if (!ms) {
        return;
    }

    sleep_ms(ms);
}

extern "C" void delayMicroseconds( unsigned int usec ) {
    if (!usec) {
        return;
    }
    sleep_us(usec);
}

extern "C" void yield() {
    // NOOP
}

extern "C" unsigned long long millis()
{
    unsigned long m = to_ms_since_boot(get_absolute_time());
    //casp
    unsigned int diff = 0;
    if(m < g_casp_milli_prev_tick)
        diff = ((0xffffffff-g_casp_milli_prev_tick)+m);
    else
        diff = m-g_casp_milli_prev_tick;
    g_casp_milli_tick_counter += diff;
    g_casp_milli_prev_tick = m;
    return g_casp_milli_tick_counter;
}

extern "C" unsigned long long micros()
{
    unsigned long m = to_us_since_boot(get_absolute_time());//time_us_32();
    //casp
    unsigned int diff = 0;
    if(m < g_casp_micro_prev_tick)
        diff = ((0xffffffff-g_casp_micro_prev_tick)+m);
    else
        diff = m-g_casp_micro_prev_tick;
    g_casp_micro_tick_counter += diff;
    g_casp_micro_prev_tick = m;
    return g_casp_micro_tick_counter;
}
